/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include "fitsio.h"
#include "imaging_data.h"
#include "imaging_fdata.h"
#include "oirSupport.h"
#ifdef HP
#define finite(x) isfinite(x)
#endif

/**********************************************************
 * Usage:
 *    oir1dCompressData "inFiles" maskFile outFile
 *
 *    inFiles is a blank-separated list of input files
************************************************************** */

int main (int argc, char** argv) {

   char **fileList;
   int nFiles, iFile;
   int nRegion, iRegion;
   int maxstep;
   int ndit;
   int i,ierr;
   long nRows, iRow, oRow;
   int nx, ny, ix;
   int nxList[12], nyList[12];
   long ipix;
   short *inPix;
   float *maskPix;
   float *maskImages[12];
   int isMask;
   int iKey;

   char outFileName[130];
   char history[80];
   int  iHistory;
   char *bang = "!";

   fitsfile *inFile;
   fitsfile *outFile = NULL;
   imaging_data_table_def *idata;
   imaging_fdata_table_def *odata;
   imaging_detector_table_def *inDetectorDef;
   imaging_detector_table_def *outDetectorDef;
   imaging_data_row *inData;
   imaging_fdata_row *outData;

   printf("      ******   oir1dCompressData   ******\n");
   if (argc < 4) {
     printf("USAGE: %s \"inFiles\" maskFile outFile\n",argv[0]);
     printf("	    inFiles is a blank-separated list of input files\n");
     return 10;
   }
/*        break up input file list and print what you find */
   fileList = oirStrSplit(argv[1], &nFiles);
   printf("   Reading input data from %i files:\n",nFiles);
   for (i=0; i<nFiles; i++) {printf("      %s \n", fileList[i]);}
   ierr = 0;
/*
      open first file and get detector info; then close file
*/
   fits_open_file (&inFile, fileList[0], READONLY, &ierr);
   if (ierr != 0) {
      printf("fits openfile of %s gave error %i \n",fileList[0],
         ierr);
       printf("%s \n",strerror(errno));
      return 1;
   }
   i = fits_read_key(inFile, TINT, "HIERARCH ESO DET NDIT", &ndit,
      0, &ierr);
/*   printf("NDIT = %i\n", ndit); */
   outDetectorDef = read_imaging_detector_table(inFile, &ierr);
   idata          = open_imaging_data_table(inFile, &ierr);
   inData         = make_imaging_data_row(idata);
   nRegion        = idata->nregion;
   for (iRegion=0;iRegion<nRegion;iRegion++) {
      nxList[iRegion] = outDetectorDef->naxis[iRegion][0];
      nyList[iRegion] = outDetectorDef->naxis[iRegion][1];
   }
/*
      get mask data
*/
   if (0 == strlen(argv[2])) isMask = (1==0);
   else {
      oirGetMaskImages(argv[2], maskImages, nRegion, nxList, nyList, &ierr);
      isMask = (ierr == 0);
   }
   if (isMask) printf("   Using maskfile %s\n", argv[2]);
   else if (ierr !=0) {
      fprintf(stderr, "maskfile %s gave error %i\n",argv[2], ierr);
      return 1;
   }

/*
      need maxstep for output table definition
*/
   iKey = table_find_key(idata->def, "MAXSTEP");
   if (iKey>0)  table_get_keyword (idata->def, iKey, (void*)&maxstep,
      &ierr);
/*
      reset output data sizes to reflect compression
*/
   for (iRegion=0; iRegion<nRegion; iRegion++) {
      outDetectorDef->naxis[iRegion][1] = 1;
   }
/*
      create output data table
*/
   strcpy(outFileName, bang);
   strcat(outFileName, argv[3]);

   fits_create_file (&outFile, outFileName, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error opening fits output %s %i\n", outFileName, ierr);
      fprintf (stderr, "%s \n", strerror(errno), ierr);
      return 1;
      }
/*
      copy main header from input to output and close input
      also write names of input files as history records
      if longer than 72 char, only write last 72 char
*/
   i = fits_movabs_hdu(inFile, 1, 0, &ierr);
   i = fits_copy_header(inFile, outFile, &ierr);
   for (iFile=0; iFile<nFiles; iFile++) {
      snprintf(history,80,"oir1dCompressData INPUT file %i: ", iFile+1);
      i=fits_write_history(outFile, history, &ierr);
      iHistory = strlen(fileList[iFile]);  /* length of filename */
      iHistory = iHistory - 72; /* only 72 characters in a history record */
      if (iHistory <0) iHistory = 0;
      snprintf(history,80,"%s", fileList[iFile]+iHistory);
      i=fits_write_history(outFile, history, &ierr);
   }
      snprintf(history,80,"oir1dCompressData OUTPUT file: ");
      i=fits_write_history(outFile, history, &ierr);
      iHistory = strlen(outFileName);  /* length of filename */
      iHistory = iHistory - 72; /* only 72 characters in a history record */
      if (iHistory <1) iHistory = 1; /* supress leading bang ! symbol */
      snprintf(history,80,"%s", outFileName+iHistory);
      i=fits_write_history(outFile, history, &ierr);
   if (isMask) {
      snprintf(history,80, "oir1dCompressData MASKFILE is: ");
      i=fits_write_history(outFile, history, &ierr);
      iHistory = strlen(argv[2]);  /* length of filename */
      iHistory = iHistory - 72; /* only 72 characters in a history record */
      if (iHistory <0) iHistory = 0;
      snprintf(history,80,"%s", argv[2]+iHistory);
      i=fits_write_history(outFile, history, &ierr);
   }
   i = fits_close_file(inFile, &ierr);

   odata = make_imaging_fdata_table_def (outFile,
      outDetectorDef, maxstep, &ierr);
   if (ierr !=0) {
      fprintf (stderr, "error creating data definition\n");
      return 1;
      }
   outData = make_imaging_fdata_row(odata);
   oRow = 1;
/*
    loop over input files
*/
   for (iFile=0; iFile<nFiles; iFile++) {
      fits_open_file (&inFile, fileList[iFile], READONLY, &ierr);
      if (ierr != 0) {
         fprintf(stderr,"fits openfile of %s gave error %i\n",fileList[iFile],
            ierr);
         fprintf(stderr,"%s \n",strerror(errno));
         return 1;
      }
      inDetectorDef = read_imaging_detector_table(inFile, &ierr);
      idata = open_imaging_data_table(inFile, &ierr);
      i = fits_get_num_rows (inFile,  &nRows, &ierr);
/* suppress autoscaling to avoid overflow */
      for (i=0; i<idata->nregion; i++)
         fits_set_tscale(inFile, 1+idata->data_col[i], 1.0, 0.0, &ierr);
/*
         loop over input rows
*/
      for (iRow=1; iRow<=nRows; iRow++) {
         i = read_imaging_data_table(inFile, idata, inData, iRow, &ierr);

/*
            check for invalid time; this indicates EXPREP failure and
            the OPD values are also invalid.  Set these to zero
            but leave time as invalid to serve as flag
*/
         if (0 == finite(inData->time)) {
            inData->opd[0] = 0.;
            inData->opd[1] = 0.;
            inData->local_opd[0] = 0.;
            inData->local_opd[1] = 0.;
         }
/*
            copy nonData info
*/
         outData->time = inData->time;
         outData->opd[0] = inData->opd[0];
         outData->opd[1] = inData->opd[1];
         outData->local_opd[1] = inData->local_opd[1];
         outData->local_opd[0] = inData->local_opd[0];
         outData->exp = inData->exp;
         outData->offset[0] = inData->offset[0];
         outData->offset[1] = inData->offset[1];
         outData->rotation  = inData->rotation;
         outData->frame     = inData->frame;
         outData->reference = inData->reference;
         outData->step      = inData->step;
         for (i=0; i<idata->nregion; i++)
            outData->targtype[i] = inData->targtype[i];
/*
            compress real data
*/
         for (iRegion=0; iRegion<nRegion; iRegion++)  {
            nx = inDetectorDef->naxis[iRegion][0];
            ny = inDetectorDef->naxis[iRegion][1];
            for (ix=0; ix<nx; ix++) {
               outData->data[iRegion][ix] = 0.;
               inPix = (short *)inData->data[iRegion]+ix;
               if (isMask) {
                  maskPix = maskImages[iRegion]+ix;
                  for (ipix=0;ipix<ny;ipix++) {
                     outData->data[iRegion][ix] =
                        outData->data[iRegion][ix] +
                     (*(inPix)+32768.) * (*(maskPix));
                     inPix = inPix + nx;
                     maskPix  = maskPix  + nx;
                  }
               } else for (ipix=0;ipix<ny;ipix++) {
                  outData->data[iRegion][ix] =
                  outData->data[iRegion][ix] + 32768.+
                  *(inPix);
                  inPix = inPix + nx;
               }
            } /* loop over ix */
         }
/*
            write output row; write * every 1000 rows
*/
         i = write_imaging_fdata_table(outFile, odata, outData,
            oRow++, &ierr);
         if (0 == oRow - 100*(int)(oRow/100)) fprintf(stderr,"Frame: %ld\r",oRow);
      }  /* inputdata is valid for this row */
      i = fits_close_file(inFile, &ierr);
   } /* end of loop over iFile */
   printf("   number of processed frames   = %ld\n",oRow-1);
   i = close_imaging_fdata_table(outFile, odata, &ierr);
   kill_imaging_fdata_row(outData);
   kill_imaging_data_row(inData);
   kill_imaging_data_table_def (idata,1);
   for (iRegion=0;iRegion<nRegion;iRegion++) free(maskImages[iRegion]);
   return 0;
}
